# -*- coding: utf-8 -*-

"""
Author: Alexandra Moine-Franel
Date: March 2018
Version: 2



IDENTIFY PROTEIN-PROTEIN COMPLEXES FROM PDBe


Usage: [Script.py] [PDBe]
------------------------------------------------------------------
Argument:
	[PDBe]:
    Input (json format)
    PDBe (with [assembly_composition] and [assembly_type] information included)

Return:
	[list(PPC)]:
    Output (txt format)
    Protein/protein complex PDB IDs
	
	[json(PPC)]:
	Output (json format)
	Protein/protein complex information
	
	
Warning:
------- 
PDBs with unique and repeated assembly composition annotation are both included!
[unique annotation: 'protein/protein complex']
[repeated annotation: 'protein/protein complex, protein/protein complex']
"""

# =============================================================================

import os
import sys
import argparse
import logging
import json

# =============================================================================

LOG = logging.getLogger('Get PPC PDB IDs from PDBe')
FILENAME = 'PPC-DIMER_3-26-18PDBe'

XRAY = ['X-ray diffraction', 'X-ray powder diffraction']

RESOLUTION = 3.5
DIFFR = 0.05

# =============================================================================

def main(pdbe):
	with open(pdbe) as infile:
		structure = json.load(infile)
		dimer = get_dimer(get_ppc(structure))
		dimer_filt = filter_rfactor(filter_resolution(filter_annotation(dimer)))
		to_json(dimer_filt, FILENAME)
		to_txt(get_id(dimer_filt), FILENAME)

	if os.path.exists('{}.txt'.format(FILENAME)):
		LOG.info('Finished!')
	else:
		LOG.error('Warning! [Output] Not found!')
		sys.exit(1)



def get_ppc(pdbe):
	ppc_unique = []
	ppc_repeated = []
	for i in xrange(len(pdbe)):
		if 'assembly_composition' in pdbe[i].keys():
			assembly_composition = pdbe[i]['assembly_composition']
			if len(assembly_composition) == 1 and \
			assembly_composition == ['protein/protein complex']:
				ppc_unique.append(pdbe[i])
			elif len(assembly_composition) > 1:
				if assembly_composition.count('protein/protein complex') >= 1 and \
				assembly_composition.count(
				assembly_composition[0]) == len(assembly_composition):
					ppc_repeated.append(pdbe[i])
	ppc_all = ppc_unique + ppc_repeated
	return ppc_all


def get_dimer(pdbe):
	dimer_unique = []
	dimer_repeated = []
	for i in xrange(len(pdbe)):
		if 'assembly_type' in pdbe[i].keys():
			assembly_type = pdbe[i]['assembly_type']
			if len(assembly_type) == 1 and \
			assembly_type == ['dimer']:
				dimer_unique.append(pdbe[i])
			elif len(assembly_type) > 1:
				if assembly_type.count('dimer') >= 1 and \
				assembly_type.count(assembly_type[0]) == len(assembly_type):
					dimer_repeated.append(pdbe[i])
	dimer_all = dimer_unique + dimer_repeated
	return dimer_all


def filter_annotation(pdbe):
	annotation = []
	for i in xrange(len(pdbe)):
		if len(pdbe[i]['experimental_method']) == 1:
			annotation.append(pdbe[i])
	return annotation

def filter_resolution(pdbe):
	quality = []
	for i in xrange(len(pdbe)):
		if len([e for e in pdbe[i]['experimental_method'] if e in XRAY]) == 1:
			if 'resolution' in pdbe[i].keys() and \
			float(pdbe[i]['resolution']) <= RESOLUTION:
				quality.append(pdbe[i])
	return quality

def filter_rfactor(pdbe):
	quality = []
	for i in xrange(len(pdbe)):
		if len([e for e in pdbe[i]['experimental_method'] if e in XRAY]) == 1:
			if 'r_free' in pdbe[i].keys() and 'r_factor' in pdbe[i].keys():
				if float(pdbe[i]['r_free'] - pdbe[i]['r_factor']) <= abs(DIFFR):
					quality.append(pdbe[i])
	return quality


def get_id(pdbe):
	pdb_id = []
	for i in xrange(len(pdbe)):
		pdb_id.append(pdbe[i]['pdb_id'])
	return pdb_id


def to_json(data, outfile):
	with open('{}.json'.format(outfile), 'wb') as jsonfile:
		json.dump(data, jsonfile)


def to_txt(data, outfile):
	with open('{}.txt'.format(outfile), 'wb') as txtfile:
		for pdb_id in data:
			txtfile.write(pdb_id + '\n')



def setlogger():
	LOG.setLevel(logging.INFO)
	ch = logging.StreamHandler()
	ch.setLevel(logging.INFO)
	formatter = logging.Formatter('%(asctime)s - %(name)s - %(funcName)s - \
	%(levelname)s - %(message)s')
	ch.setFormatter(formatter)
	LOG.addHandler(ch)

# =============================================================================

if __name__ == "__main__":
	parser = argparse.ArgumentParser(description = 'Get protein/protein \
	complex PDB IDs from PDBe')
	parser.add_argument('pdbe', help = 'Input [.json file]: \
	PBDe (with assembly and structure quality information included')
	options = parser.parse_args()
	setlogger()
	main(options.pdbe)
