# -*- coding: utf-8 -*-

"""
Author: Alexandra Moine-Franel
Date: February 2018
Version: 1



IDENTIFY PROTEIN-PROTEIN COMPLEXES FROM PDBe


Usage: [Script.py] [PDBe]
------------------------------------------------------------------
Argument:
	[PDBe]:
    Input (json format)
    PDBe (with assembly information included)

Return:
	[list(PPC)]:
    Output (txt format)
    Protein/protein complex PDB IDs
	
	[json(PPC)]:
	Output (json format)
	Protein/protein complex information
	
	
Warning:
------- 
PDBs with unique and repeated assembly composition annotation are both included!
[unique annotation: 'protein/protein complex']
[repeated annotation: 'protein/protein complex, protein/protein complex']
"""

# =============================================================================

import argparse
import logging
import json
import pandas as pd

# =============================================================================

LOG = logging.getLogger('Get protein/protein complex PDB IDs from PDBe')

# =============================================================================

def setlogger():
    LOG.setLevel(logging.INFO)
    ch = logging.StreamHandler()
    ch.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(funcName)s - %(levelname)s - %(message)s')
    ch.setFormatter(formatter)
    LOG.addHandler(ch)



def main(pdbe):
	structure = pd.read_json(pdbe, orient = 'columns')
	ppc_to_txt(get_ppc_id(structure))
	ppc_to_json(get_ppc_data(structure))



def get_ppc_id(pdbe):
	ppc_unique = []
	ppc_repeated = []
	for i in xrange(len(pdbe['grouped']['pdb_id']['groups'])):
		assembly_composition = pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0]['assembly_composition']
		if len(assembly_composition) == 1 and assembly_composition == ['protein/protein complex']:
			ppc_unique.append(pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0]['pdb_id'])
		elif len(assembly_composition) > 1:
			if assembly_composition.count('protein/protein complex') >= 1 and \
			assembly_composition.count(assembly_composition[0]) == len(assembly_composition):
				ppc_repeated.append(pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0]['pdb_id'])
	ppc_all = ppc_unique + ppc_repeated
	return ppc_all



def get_ppc_data(pdbe):
	ppc_unique_data = []
	ppc_repeated_data = []
	for i in xrange(len(pdbe['grouped']['pdb_id']['groups'])):
		assembly_composition = pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0]['assembly_composition']
		if len(assembly_composition) == 1 and assembly_composition == ['protein/protein complex']:
			ppc_unique_data.append(pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0])
		elif len(assembly_composition) > 1:
			if assembly_composition.count('protein/protein complex') >= 1 and \
			assembly_composition.count(assembly_composition[0]) == len(assembly_composition):
				ppc_repeated_data.append(pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0])
	ppc_all_data = ppc_unique_data + ppc_repeated_data
	return ppc_all_data



def ppc_to_txt(ppc):
	with open('PPC_2-20-18PDBe.txt', 'wb') as txtfile:
		for pdb_id in ppc:
			txtfile.write(pdb_id + '\n')



def ppc_to_json(ppc):
	with open('PPC_2-20-18PDBe.json', 'wb') as jsonfile:
		json.dump(ppc, jsonfile)



# ==============================================================================

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description = 
					'Get protein/protein complex PDB IDs from PDBe')
    parser.add_argument('pdbe', help = 'Input [.json file]: \
					PBDe (with assembly information included')
    options = parser.parse_args()
    setlogger()
    main(options.pdbe)
