# -*- coding: utf-8 -*-

"""
Author: Alexandra Moine-Franel
Date: February 2018
Version: 2



IDENTIFY PROTEIN STRUCTURE WITH COMPOUNDS (with or without including ions)
INVOLVED IN PROTEIN-PROTEIN INTERACTION FROM PDBe


Usage: [Script.py] [PDBe]
------------------------------------------------------------------
Argument:
	[PDBe]:
    Input (json format)
    PDBe (with [assembly_composition], [compound_id] and [uniprot_id] included)

Return:
	[list(protein)]:
    Output (txt format)
    Proteins with compounds associated with a protein/protein complex PDB IDs
	
	[json(protein)]:
    Output (json format)	
    Proteins with compounds associated with a protein/protein complex dataset

	Compounds:
	---------
	** EXAMPLE: [u'FHB : 3-FLUORO-4-HYDROXYBENZOIC ACID']
			or  [u'FHB : 3-FLUORO-4-HYDROXYBENZOIC ACID', u'FE : FE (III) ION']



    [list(protein)]:
    Output (txt format)
    Proteins with compounds associated with a protein/protein complex PDB IDs
    IONS ONLY EXCLUDED

    [json(protein)]:
    Output (json format)        
    Proteins with compounds associated with a protein/protein complex dataset
    IONS ONLY EXCLUDED

	Compounds:
	---------
	** EXAMPLE: [u'FE : FE (III) ION']
	


!Warning!:
------- 
PDBs with unique and repeated assembly composition annotation are both included!
[unique annotation: 'protein/protein complex']
[repeated annotation: 'protein/protein complex, protein/protein complex']
"""

# =============================================================================

import argparse
import logging
import json
import pandas as pd

# =============================================================================

LOG = logging.getLogger('Proteins associated with a PPC - Compounds')

# =============================================================================

def setlogger():
    LOG.setLevel(logging.INFO)
    ch = logging.StreamHandler()
    ch.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(funcName)s - %(levelname)s - %(message)s')
    ch.setFormatter(formatter)
    LOG.addHandler(ch)


def main(pdbe):
	structure = pd.read_json(pdbe, orient = 'columns')

	# Get protein/protein complex [assembly_composition]
	ppc = get_ppc_data(structure)

	# Get protein structure [assembly_composition] with 'compounds'
	prot_compound = get_compound_id(get_prot_data(structure))

	# Collect PPC UniProt
	ppc_uniprot = get_uniprot(ppc)

	# Identify protein structure with compounds associated to PPC 
	to_txt(get_id_prot_compound_ppc(prot_compound, ppc_uniprot))
	to_json(get_data_prot_compound_ppc(prot_compound, ppc_uniprot))

	
	# Get protein structure [assembly_composition] with compounds (ions only not included)
	ion = get_ion(get_compound_name(prot_compound))
	prot_compound_without_ions = get_compound_without_ion(prot_compound, ion)
	to_txt_ion(get_id_prot_compound_ppc(prot_compound_without_ions, ppc_uniprot))
	to_json_ion(get_data_prot_compound_ppc(prot_compound_without_ions, ppc_uniprot))




def get_ppc_data(pdbe):
	ppc_unique_data = []
	ppc_repeated_data = []
	for i in xrange(len(pdbe['grouped']['pdb_id']['groups'])):
		assembly_composition = pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0]['assembly_composition']
		if len(assembly_composition) == 1 and assembly_composition == ['protein/protein complex']:
			ppc_unique_data.append(pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0])
		elif len(assembly_composition) > 1:
			if assembly_composition.count('protein/protein complex') >= 1 and \
			assembly_composition.count(assembly_composition[0]) == len(assembly_composition):
				ppc_repeated_data.append(pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0])
	ppc_all_data = ppc_unique_data + ppc_repeated_data
	return ppc_all_data



def get_prot_data(pdbe):
	prot_unique_data = []
	prot_repeated_data = []
	for i in xrange(len(pdbe['grouped']['pdb_id']['groups'])):
		assembly_composition = pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0]['assembly_composition']
		if len(assembly_composition) == 1 and assembly_composition == ['protein structure']:
			prot_unique_data.append(pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0])
		elif len(assembly_composition) > 1:
			if assembly_composition.count('protein structure') >= 1 and \
			assembly_composition.count(assembly_composition[0]) == len(assembly_composition):
				prot_repeated_data.append(pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0])
	prot_all_data = prot_unique_data + prot_repeated_data
	return prot_all_data



def get_compound_id(data):
	'''
	Get 3D structure with compound [compound_id]
	--------------------------------------------------
	Argument:
		[json]: protein structures (dataset)

	Return:
		[json]: protein structures with compounds (dataset)
	'''
	compound = []
	for i in xrange(len(data)):
                if 'compound_id' in data[i].keys():
					compound.append(data[i])
	return compound



def get_uniprot(data):
	'''
	Get UniProt
	-------------------------------------------------
	Argument:
		[json]: protein/protein complexes (dataset)

	Return:
		[list]: protein/protein complexes Uniprot IDs
	'''
	uniprot = []
	for i in xrange(len(data)):
		if 'uniprot_id' in data[i].keys():
			uniprot.extend(data[i]['uniprot_id'])
	return uniprot


def get_id_prot_compound_ppc(data, uniprot):
	'''
	Get PDB IDs only of protein structure with compounds associated with a protein/protein complex
	-----------------------------------------------------------------------------------------------
	Arguments:
		[json]: protein structures with compounds (dataset)
		[list]: protein/protein complex uniprot id

	Return:
		[list]: protein structure with compounds associated with a protein/protein complex PDB IDs
	''' 
	prot_compound_pcc = []
	for i in xrange(len(data)):
		if 'uniprot_id' in data[i].keys():
			if len([e for e in data[i]['uniprot_id'] if e in uniprot]) >= 1:
				prot_compound_pcc.append(data[i]['pdb_id'])
	return prot_compound_pcc
				


def get_data_prot_compound_ppc(data, uniprot):
	'''
	Get data only of protein structure with compounds associated with a protein/protein complex
	-----------------------------------------------------------------------------------------------
	Arguments:
		[json]: protein structures with compounds (dataset)
		[list]: protein/protein complex uniprot id

	Return:
		[json]: protein structure with compounds associated with a protein/protein complex (dataset)
	''' 
	prot_compound_pcc_data = []
	for i in xrange(len(data)):
		if 'uniprot_id' in data[i].keys():
			if len([e for e in data[i]['uniprot_id'] if e in uniprot]) >= 1:
					prot_compound_pcc_data.append(data[i])
	return prot_compound_pcc_data




def get_compound_name(data):
	'''
	Get protein compound names
	--------------------------------------------------
	Argument:
		[json]: protein structures with compounds (dataset)

	Return:
		[list]: compound names
		** EXAMPLE: [u'FHB : 3-FLUORO-4-HYDROXYBENZOIC ACID', u'FE : FE (III) ION']
	'''
	compound = []
	for i in xrange(len(data)):
		if 'compound_name' in data[i].keys():
			compound.extend(data[i]['compound_name'])
	return compound

		

def get_ion(compound_name):
	'''
	Get protein ions
	--------------------------------------------------
	Argument:
		[list]: compound names
		** EXAMPLE: [u'FHB : 3-FLUORO-4-HYDROXYBENZOIC ACID', u'FE : FE (III) ION']

	Return:
		[list]: ions
		** EXAMPLE: ['FE']
	'''
	ion = []
	ion_unique = []
	for i in xrange(len(compound_name)):
		if 'ION' in compound_name[i].split(' :')[1].split(' '):
			ion.append(compound_name[i].split(' :')[0])
	for j in ion:
		if j not in ion_unique:
			ion_unique.append(j)
	print ion_unique
	return ion_unique



def get_compound_without_ion(data, ions):
	'''
	Get 3D structure with compound [compound_id], ion excluded
	----------------------------------------------------------
	Arguments:
		[json]: protein structures (dataset)
		[list]: ions

	Return:
		[json]: protein structures with compounds != ions (dataset)
				
	!Warning!
	---------
	Protein structures either with only compounds, or compounds and ions are included
	** EXAMPLE: [u'FHB : 3-FLUORO-4-HYDROXYBENZOIC ACID']
			or  [u'FHB : 3-FLUORO-4-HYDROXYBENZOIC ACID', u'FE : FE (III) ION']

	Only protein structures with only ions are excluded
	** EXAMPLE: [u'FE : FE (III) ION']				
	'''
	compound_without_ion = []
	compound_with_ion = []
	for i in xrange(len(data)):
		if 'compound_id' in data[i].keys():
			if len([e for e in data[i]['compound_id'] if e in ions]) == 0:
				compound_without_ion.append(data[i])
			elif len([e for e in data[i]['compound_id'] if e in ions]) >= 1:
				if len([e for e in data[i]['compound_id'] if e in ions]) != len(data[i]['compound_id']):
					compound_with_ion.append(data[i])
	compound_ion = compound_without_ion + compound_with_ion				
	return compound_ion



def to_txt(index):
	with open('PROT-COMPOUNDS_PPC_2-20-18PDBe.txt', 'wb') as txtfile:
		for i in index:
			txtfile.write(i + '\n')

def to_json(index):
	with open('PROT-COMPOUNDS_PPC_2-20-18PDBe.json', 'wb') as jsonfile:
		json.dump(index, jsonfile)

def to_txt_ion(index):
	with open('PROT-COMPOUNDS-noIONS_PPC_2-20-18PDBe.txt', 'wb') as txtfile:
		for i in index:
			txtfile.write(i + '\n')

def to_json_ion(index):
	with open('PROT-COMPOUNDS-noIONS_PPC_2-20-18PDBe.json', 'wb') as jsonfile:
		json.dump(index, jsonfile)

# ==============================================================================

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description = 
					'IDENTIFY PROTEIN STRUCTURE WITH COMPOUNDS INVOLVED IN PROTEIN-PROTEIN INTERACTION FROM PDBe')
    parser.add_argument('pdbe', help = 'Input [.json file]: \
					PBDe (with [assembly_composition], [compound_id] and [uniprot_id] included)')
    options = parser.parse_args()
    setlogger()
    main(options.pdbe)
