# -*- coding: utf-8 -*-

"""
Author: Alexandra Moine-Franel
Date: January 2018
Version: 2



CALCULATE 2D/3D POCKET DESCRIPTORS FROM VOLSITE


Usage: [Script.py] [PDB_descriptor.txt] [PDB_2D-3Ddescriptors.csv]
------------------------------------------------------------------

[PDB_descriptor.txt]:
    Input (txt format)
    Pocket descriptors calculated by VolSite
	
[PDB_2D-3Ddescriptors.csv]:
    Output (csv format)
    Pocket descriptors including:
    * 89 descriptors from VolSite
    * 10 descriptors using a linear combination of VolSite descriptors
      (Kuenemann, 2016 Scientific Reports)
    * 10 geometric descriptors from RDKit toolkit
      (http://www.rdkit.org/docs/Overview.html, version 2017.09.1)
"""

# =============================================================================

import os
import sys
import argparse
import logging
import csv
import pandas as pd
from rdkit import Chem
from rdkit.Chem import Descriptors3D

# =============================================================================

LOG = logging.getLogger("CALCULATE 2D/3D POCKET DESCRIPTORS FROM VOLSITE")


VOLSITE_HEADER = [
    'Volume', 'CZ', 'CA', 'O', 'OD1', 'OG', 'N', 'NZ', 'DU',
    'CZ40', 'CZ40-50', 'CZ50-60', 'CZ60-70', 'CZ70-80', 'CZ80-90', 'CZ90-100', 'CZ100-110', 'CZ110-120', 'CZ120',
    'CA40', 'CA40-50', 'CA50-60', 'CA60-70', 'CA70-80', 'CA80-90', 'CA90-100', 'CA100-110', 'CA110-120', 'CA120',
    'O40', 'O40-50', 'O50-60', 'O60-70', 'O70-80', 'O80-90', 'O90-100', 'O100-110', 'O110-120', 'O120',
    'OD140', 'OD140-50', 'OD150-60', 'OD160-70', 'OD170-80', 'OD180-90', 'OD190-100', 'OD1100-110', 'OD1110-120',
    'OD1120',
    'OG40', 'OG40-50', 'OG50-60', 'OG60-70', 'OG70-80', 'OG80-90', 'OG90-100', 'OG100-110', 'OG110-120', 'OG120',
    'N40', 'N40-50', 'N50-60', 'N60-70', 'N70-80', 'N80-90', 'N90-100', 'N100-110', 'N110-120', 'N120',
    'NZ40', 'NZ40-50', 'NZ50-60', 'NZ60-70', 'NZ70-80', 'NZ80-90', 'NZ90-100', 'NZ100-110', 'NZ110-120', 'NZ120',
    'DU40', 'DU40-50', 'DU50-60', 'DU60-70', 'DU70-80', 'DU80-90', 'DU90-100', 'DU100-110', 'DU110-120', 'DU120',
    'POCKET'
]

DESC_COMBINATION = {
    'T40': ['CZ40', 'CA40', 'O40', 'OD140', 'OG40', 'N40', 'NZ40', 'DU40'],
    'T40-50': ['CZ40-50', 'CA40-50', 'O40-50', 'OD140-50', 'OG40-50', 'N40-50', 'NZ40-50', 'DU40-50'],
    'T50-60': ['CZ50-60', 'CA50-60', 'O50-60', 'OD150-60', 'OG50-60', 'N50-60', 'NZ50-60', 'DU50-60'],
    'T60-70': ['CZ60-70', 'CA60-70', 'O60-70', 'OD160-70', 'OG60-70', 'N60-70', 'NZ60-70', 'DU60-70'],
    'T70-80': ['CZ70-80', 'CA70-80', 'O70-80', 'OD170-80', 'OG70-80', 'N70-80', 'NZ70-80', 'DU70-80'],
    'T80-90': ['CZ80-90', 'CA80-90', 'O80-90', 'OD180-90', 'OG80-90', 'N80-90', 'NZ80-90', 'DU80-90'],
    'T90-100': ['CZ90-100', 'CA90-100', 'O90-100', 'OD190-100', 'OG90-100', 'N90-100', 'NZ90-100', 'DU90-100'],
    'T100-110': ['CZ100-110', 'CA100-110', 'O100-110', 'OD1100-110', 'OG100-110', 'N100-110', 'NZ100-110', 'DU100-110'],
    'T110-120': ['CZ110-120', 'CA110-120', 'O110-120', 'OD1110-120', 'OG110-120', 'N110-120', 'NZ110-120', 'DU110-120'],
}

DESC_GEOM = [
    'PMI1', 'PMI2', 'PMI3', 'NPR1', 'NPR2', 'Rgyr',
    'Asphericity', 'SpherocityIndex', 'Eccentricity', 'InertialShapeFactor'
]

# =============================================================================

def setlogger():
    LOG.setLevel(logging.INFO)
    ch = logging.StreamHandler()
    ch.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(funcName)s - \
								   %(levelname)s - %(message)s')
    ch.setFormatter(formatter)
    LOG.addHandler(ch)


def main(volsitedescriptor, pocketdescriptor):
    volsite_csv = descriptor_volsite(volsitedescriptor)
    combination_csv = descriptor_combination(volsite_csv)
    volsite_combination_csv = "{}volsite+combinaison.csv".format(
							  volsite_csv.rsplit("volsite.csv")[0])
    merge_csv(volsite_csv, combination_csv, volsite_combination_csv)
    geometric_csv = descriptor_geometric(volsitedescriptor)
    merge_csv(volsite_combination_csv, geometric_csv, pocketdescriptor)

    if os.path.exists(pocketdescriptor):
        os.remove(volsite_csv)
        os.remove(combination_csv)
        os.remove(volsite_combination_csv)
        os.remove(geometric_csv)
        LOG.info('Finished!')
    else:
        LOG.error('Error: 2D/3D Pocket Descriptor file not found')
        sys.exit(1)


def descriptor_volsite(txtfile):
    """
    Convert VolSite output file from txt format to csv format
    (descriptor headers added)
    ---------------------------------------------------------
    Argument:
        [file]: VolSite descriptors (txt format)

    Return:
        [file]: VolSite descriptors (csv format)
                with descriptor headers
    """
    try:
        with open(txtfile, 'rb') as infile:
            volsite_csv = "{}_volsite.csv".format(os.path.splitext(txtfile)[0])
            with open(volsite_csv, 'wb') as outfile:
                in_text = csv.reader(infile, delimiter=' ')
                out_csv = csv.writer(outfile)
                out_csv.writerow(VOLSITE_HEADER)
                for line in in_text:
                    out_csv.writerow(line)
    except IOError, e:
        LOG.error('{}'.format(e))
        sys.exit(1)
    return volsite_csv


def descriptor_combination(volsite_descriptor_csv):
    """
    Calculate and write linear combinations of VolSite descriptors to csv file
   (Kuenemann, 2016 Scientific Reports)
    ---------------------------------------------------------------------------
    Argument:
        [file]: VolSite descriptors (csv format)

    Return:
        [file]: linear combinations of VolSite descriptors (csv format)
    """
    with open(volsite_descriptor_csv, 'rb') as infile:
        volsite = csv.reader(infile, delimiter = ',')
        descriptors = volsite.next()

        # Create a dictionnary
        # key(descriptor): values(list of all pocket values)
        data = {}
        # Count the number of pockets
        numbpocket = 0
        for header in descriptors:
            data[header] = []
        for pocket in volsite:
            numbpocket += + 1
            for header, value in zip(descriptors, pocket):
                data[header].append(value)
        combination_csv = "{}combination.csv".format(
						  volsite_descriptor_csv.rsplit("volsite.csv")[0])
        with open(combination_csv, 'wb') as outfile:
            combivolsite = csv.writer(outfile, delimiter = ',', 
									  lineterminator = '\n')
            # Descriptor headers and values have to be in variables
            # to write them in a new file column by column (sep = comma)
            descname = []
            cdata = []
            for dch in DESC_COMBINATION.keys():
                # dch = descriptor combination header (column)
                bsum = []
                for p in range(numbpocket):
                    # p = pocket (row)
                    buriedness_values = []
                    for colname in DESC_COMBINATION[dch]:
                        # Get buriedness value by descriptors
                        buriedness_values.append(float(data[colname][p]))
                    # Get sum of buriedness value by descriptor combinations
                    bsum.append(sum(buriedness_values))
                descname.append(dch)
                cdata.append(bsum)
            combivolsite.writerow(descname)
            datatransposed = zip(*cdata)
            for row in datatransposed:
                combivolsite.writerow(row)
    return combination_csv


def descriptor_geometric(txtfile):
    """
    Write geometric descriptor from the negative image of binding pockets 
    detected by VolSite to csv file
    ---------------------------------------------------------------------------
    Argument:
        [file]: VolSite descriptors (txt format)

    Return:
        [file]: shape descriptors (csv format)
    """
    with open(txtfile, 'rb') as infile:
        pocketvolsite = csv.reader(infile, delimiter = ',')
        # Get the number of pockets
        numbpocket = 0
        for pocket in pocketvolsite:
            numbpocket += + 1
        geometric_csv = "{}geometric.csv".format(os.path.splitext(txtfile)[0])
        with open(geometric_csv, 'wb') as outfile:
            shape = csv.writer(outfile, delimiter = ',', lineterminator = '\n')
            shape.writerow(DESC_GEOM)
            # Calculate geometric descriptors pocket by pockets
            for cavity in range(1, numbpocket + 1):
				pocket_mol2 = "{}_CAVITY_N{}_ALL.mol2".format(
							  geometric_csv.rsplit("_")[0], cavity)	
				replace_mol2atom(pocket_mol2)
				shape.writerow(calculate_shape_descriptor(
				"temp_{}_CAVITY_N{}_ALL.mol2".format(
				geometric_csv.rsplit("_")[0], cavity)))
				os.remove("temp_{}_CAVITY_N{}_ALL.mol2".format(
				geometric_csv.rsplit("_")[0], cavity))
	return geometric_csv
		

def replace_mol2atom(cavity):
    """
    Replace the atom name assigned by VolSite to the probes by 'C' (carbone)
	
	Warning 1
	Fixing the RDKit error: 'atom' with a degree > 1
	
	Warning 2
	RDKit needs the output file [PDB_CAVITY_Nx_ALL_temp.mol2]
	NOT the link
	------------------------------------------------------------------
    Argument:
        [file]: negative image of a binding pocket (mol2 format)

    Return:
        [file]: negative image of a binding pocket with carbone as atom name
				(mol2 format, temporary file used as input by RDKit)
    """
    header = True
    parsing = False	
    with open(cavity, 'rb') as infile:
	with open("temp_{}.mol2".format(cavity.rsplit('.')[0]), 'wb') as tempf:
		for line in infile:
			if line.startswith('@<TRIPOS>ATOM'):
				tempf.write(line)# + '\n')
				header = False
				parsing = True
				continue
			elif line.startswith('@<TRIPOS>BOND'):
				parsing = False
					
			if header:
				tempf.write(line)
			if parsing:
				tempf.write(line.replace(line.split()[5], 'C'))
	#return tempf
	
			
def calculate_shape_descriptor(cavity):
    """
    Calculate geometric descriptors with RDKit 
    ------------------------------------------
    Geometric descriptor details:
    http://www.rdkit.org/Python_Docs/rdkit.Chem.Descriptors3D-module.html

    Argument:
        [file]: negative image of a binding pocket (mol2 format)

    Return:
        [list]: geometric descriptors
    """
    mol2file = Chem.MolFromMol2File(cavity, sanitize = False, removeHs = False)
    geom_val = []
    geom_val.append(Descriptors3D.PMI1(mol2file))
    geom_val.append(Descriptors3D.PMI2(mol2file))
    geom_val.append(Descriptors3D.PMI3(mol2file))
    geom_val.append(Descriptors3D.NPR1(mol2file))
    geom_val.append(Descriptors3D.NPR2(mol2file))
    geom_val.append(Descriptors3D.RadiusOfGyration(mol2file))
    geom_val.append(Descriptors3D.Asphericity(mol2file))
    geom_val.append(Descriptors3D.SpherocityIndex(mol2file))
    geom_val.append(Descriptors3D.Eccentricity(mol2file))
    geom_val.append(Descriptors3D.InertialShapeFactor(mol2file))
    return geom_val


def merge_csv(csv_infile1, csv_infile2, csv_outfile):
    """
    Concatenate two csv files
    -------------------------
    Arguments:
        [file]: input (csv format)
        [file]: input (csv format)
        [file]: output filename (csv format)
    """
    csv1 = pd.read_csv(csv_infile1)
    csv2 = pd.read_csv(csv_infile2)
    csv3 = pd.concat([csv1, csv2], axis = 1)
    return csv3.to_csv(csv_outfile, sep = ',', index = False)

# ==============================================================================

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description = 
					'Calculate 2D/3D Pocket Descriptors from VolSite')
    parser.add_argument('volsitedescriptor', help = 'Input [.txt file]: \
					Pocket descriptors calculated by VolSite')
    parser.add_argument('pocketdescriptor', help = 'Output [.csv filename]: \
					Pocket descriptors including \
					89 descriptors from VolSite, \
					10 descriptors using a combination of VolSite descriptors \
					and 10 geometric descriptors')

    options = parser.parse_args()
    setlogger()
    main(options.volsitedescriptor, options.pocketdescriptor)
