# -*- coding: utf-8 -*-

"""
Author: Alexandra Moine-Franel
Date: February 2018
Version: 4


IDENTIFY PROTEIN INTERFACE / POCKET RESIDUES

Determine protein residues at a specific distance either from protein partner, 
ligand (small molecule) or negative image of the pocket generated by VolSite.



Usage:	[Script.py] [TARGET] [PARTNER] -d (DISTANCE)
-------------------------------------------------------------------------------
[TARGET]:
	Input (.mol2 or .pdb format)
	Protein CHAIN

[PARTNER]:
	Input (.mol2 or .pdb format)
	Negative image of the pocket generated by VolSite (.mol2 format)
	OR Ligand (.pdb format)
	OR Protein (.pdb format)
	
(DISTANCE)- optionnal
	Input (float or integer)
	Distance threshold between the target and the partner
	* Distance target-protein/ligand = 5 angstroms - by defaut
	* Distance target-cavity = 3.5 angstroms - by default
	
------------------	
	
[POCKET_RESIDUES]: 
	Output (.txt format)
	'PDB-targetchain-partnerchain_distance.txt' (target-protein/ligand)
	OR 'PDB-targetchain-CAVITY_Nx_ALL_distance.txt' (target-cavity)
"""

# =============================================================================

import csv
import os
import sys
import argparse
import logging
#import time
from biopandas.mol2 import PandasMol2
from biopandas.pdb import PandasPdb

# =============================================================================

LOG = logging.getLogger("IDENTIFY PROTEIN INTERFACE / POCKET RESIDUES")

# =============================================================================

def main(target, partner, distance):
	#start = time.time()
	if target and partner:
		if os.path.splitext(target)[-1] == '.mol2' and \
		os.path.splitext(partner)[-1] == '.mol2':
			mol2target = parse_mol2(target)
			mol2partner = parse_mol2(partner)
			mol2_respocket(mol2target, mol2partner, distance)
			if os.path.exists('{}-{}_{}.txt'.format( \
			mol2target.code, mol2partner.code.rsplit('.mol2')[0], \
			distance)):
				LOG.info('Finished!')
				#end = time.time()
				#print(end - start)
			else:
				LOG.error('Warning! [Output] Not found!')
				sys.exit(1)			
		elif os.path.splitext(target)[-1] == '.pdb' and \
			os.path.splitext(partner)[-1] == '.pdb':
			pdbtarget = parse_pdb(target)
			pdbpartner = parse_pdb(partner)
			if pdbpartner.df['ATOM'].empty == True:
				pdb_resinterfacePL(pdbtarget, pdbpartner, distance)
				if os.path.exists('{}-{}-{}_{}.txt'.format(pdbtarget.code, \
				pdbtarget.df['ATOM']['chain_id'][0], pdbpartner.df['HETATM']['residue_name'][0], \
				distance)):
					LOG.info('Finished!')
					#end = time.time()
					#print(end - start)
				else:
					LOG.error('Warning! [Output] Not found!')
					sys.exit(1)
			else:
				pdb_resinterfacePP(pdbtarget, pdbpartner, distance)
				if os.path.exists('{}-{}-{}_{}.txt'.format(pdbtarget.code, \
				pdbtarget.df['ATOM']['chain_id'][0], pdbpartner.df['ATOM']['chain_id'][0], \
				distance)):
					LOG.info('Finished!')
					#end = time.time()
					#print(end - start)
				else:
					LOG.error('Warning! [Output] Not found!')
					sys.exit(1)
		else:
			LOG.error('Error! [TARGET] or [PARTNER] file not in .mol2 or .pdb format')
			sys.exit(1)
	elif target and not partner:
		parser.error('Error: [PARTNER: protein, ligand or cavity] is required')
	
	

				
def parse_mol2(mol2file):
	"""
	Parse .mol2 file 
	---------------------------------------------------------------------------
	Argument: [file]: .mol2 format
	Return: [dataframe]
	"""
	try:
		pmol = PandasMol2().read_mol2(mol2file)
	except(IOError), e:
		LOG.error('{}'.format(e))
		sys.exit(1)
	else:
		return pmol


def parse_pdb(pdbfile):
	"""
	Parse .pdb file 
	---------------------------------------------------------------------------
	Argument: [file]: .pdb format
	Return: [dataframe]
	"""
	try:
		ppdb = PandasPdb().read_pdb(pdbfile)
	except(IOError), e:
		LOG.error('{}'.format(e))
		sys.exit(1)
	else:
		return ppdb


def mol2_respocket(mol2target, mol2partner, distance):
	"""
	Identify protein target residues at a specified distance from its partner
	(i.e. the cavity negative image generated by VolSite).
	---------------------------------------------------------------------------
	Arguments:
		[file]: protein target (.mol2 format)
		[file]: negative image of the binding pocket (.mol2 format)
		[float]: distance threshold (by default, 3.5 angstroms)
	Return:
		[file]: pocket residues (.txt format)
	"""
	ppires = []
	with open('{}-{}_{}.txt'.format(mol2target.code, \
	mol2partner.code.rsplit('.mol2')[0], \
	distance), 'wb') as outfile:
		for probe in xrange(len(mol2partner.df.atom_id)):
			for atom in xrange(len(mol2target.df.atom_id)):
				if mol2target.df.subst_name[atom] not in ppires:
					xp = mol2target.df['x'][atom]
					xc = mol2partner.df['x'][probe]
					distX = x_dist(xp, xc)
					if distX < float(distance):
						yp = mol2target.df['y'][atom]
						yc = mol2partner.df['y'][probe]
						distXY = xy_dist(xp, yp, xc, yc)
						if distXY < float(distance):
							zp = mol2target.df['z'][atom]
							zc = mol2partner.df['z'][probe]
							distXYZ = euclidian_dist(xp, yp, zp, xc, yc, zc) 
							if distXYZ < float(distance):
								ppires.append(mol2target.df.subst_name[atom])
		for resp in ppires:
			outfile.write(resp + '\n')


def pdb_resinterfacePP(pdbtarget, pdbpartner, distance):
	"""
	Identify protein target residues at a specified distance from its partner
	(i.e. protein)
	---------------------------------------------------------------------------
	Arguments:
		[file]: protein target (.pdb format)
		[file]: protein partner (.pdb format)
		[float]: distance threshold (by default, 5 angstroms)
	Return:
		[file]: interface residues (.txt format)
	"""
	ppires = []
	with open('{}-{}-{}_{}.txt'.format(pdbtarget.code, \
	pdbtarget.df['ATOM']['chain_id'][0], pdbpartner.df['ATOM']['chain_id'][0], \
	distance), 'wb') as outfile:
		for atomt in xrange(len(pdbpartner.df['ATOM']['atom_number'])):
			for atomp in xrange(len(pdbtarget.df['ATOM']['atom_number'])):
				if ''.join(map(str, (pdbtarget.df['ATOM']['residue_name'][atomp], \
				pdbtarget.df['ATOM']['residue_number'][atomp]))) not in ppires:
					xp = pdbtarget.df['ATOM']['x_coord'][atomp]
					xc = pdbpartner.df['ATOM']['x_coord'][atomt]
					distX = x_dist(xp, xc)
					if distX < float(distance):
						yp = pdbtarget.df['ATOM']['y_coord'][atomp]
						yc = pdbpartner.df['ATOM']['y_coord'][atomt]
						distXY = xy_dist(xp, yp, xc, yc)
						if distXY < float(distance):
							zp = pdbtarget.df['ATOM']['z_coord'][atomp]
							zc = pdbpartner.df['ATOM']['z_coord'][atomt]
							distXYZ = euclidian_dist(xp, yp, zp, xc, yc, zc) 
							if distXYZ < float(distance):
								ppires.append(''.join(map(str, \
								(pdbtarget.df['ATOM']['residue_name'][atomp], \
								pdbtarget.df['ATOM']['residue_number'][atomp]))))
		for resp in ppires:
			outfile.write(resp + '\n')


def pdb_resinterfacePL(pdbtarget, pdbpartner, distance):
	"""
	Identify protein target residues at a specified distance from its partner
	(i.e. ligand)
	---------------------------------------------------------------------------
	Arguments:
		[file]: protein target (.pdb format)
		[file]: ligand (.pdb format)	 
		[float]: distance threshold (by default, 5 angstroms)
	Return:
		[file]: pocket residues (.txt format)
	"""
	ppires = []
	with open('{}-{}-{}_{}.txt'.format(pdbtarget.code, \
	pdbtarget.df['ATOM']['chain_id'][0], pdbpartner.df['HETATM']['residue_name'][0], \
	distance), 'wb') as outfile:
		for atomt in xrange(len(pdbpartner.df['HETATM']['atom_number'])):
			for atomp in xrange(len(pdbtarget.df['ATOM']['atom_number'])):
				if ''.join(map(str, (pdbtarget.df['ATOM']['residue_name'][atomp], \
				pdbtarget.df['ATOM']['residue_number'][atomp]))) not in ppires:
					xp = pdbtarget.df['ATOM']['x_coord'][atomp]
					xc = pdbpartner.df['HETATM']['x_coord'][atomt]
					distX = x_dist(xp, xc)
					if distX < float(distance):
						yp = pdbtarget.df['ATOM']['y_coord'][atomp]
						yc = pdbpartner.df['HETATM']['y_coord'][atomt]

						distXY = xy_dist(xp, yp, xc, yc)
						if distXY < float(distance):
							zp = pdbtarget.df['ATOM']['z_coord'][atomp]
							zc = pdbpartner.df['HETATM']['z_coord'][atomt]
							distXYZ = euclidian_dist(xp, yp, zp, xc, yc, zc) 
							if distXYZ < float(distance):
								ppires.append(''.join(map(str, \
								(pdbtarget.df['ATOM']['residue_name'][atomp], \
								pdbtarget.df['ATOM']['residue_number'][atomp]))))
		for resp in ppires:
			outfile.write(resp + '\n')


def x_dist(x1, x2):
	"""
	Calculate the distance x between two 1D points
	----------------------------------------------------------------------
	Arguments:
		[float]: coordinate x of target atom
				 coordinate x of partnet atom
	Return:
		[float]: distance 
	"""
	dist = ((x1-x2)**2) ** 0.5
	return dist


def xy_dist(x1, y1, x2, y2):
	"""
	Calculate the distance xy between two 2D points
	----------------------------------------------------------------------
	Arguments:
		[float]: coordinates xy of target atom
				 coordinates xy of partnet atom
	Return:
		[float]: distance 
	"""
	dist = ((x1-x2)**2 + (y1-y2)**2) ** 0.5	
	return dist

	
def euclidian_dist(x1, y1, z1, x2, y2, z2):
	"""
	Calculate the euclidian distance xyz between two 3D points
	----------------------------------------------------------------------
	Arguments:
		[float]: coordinates xyz of target atom
				 coordinates xyz of partnet atom
	Return:
		[float]: distance 
	"""
	dist = ((x1-x2)**2 + (y1-y2)**2 + (z1-z2)**2) ** 0.5
	return dist
	
	
	
def setlogger():
	LOG.setLevel(logging.INFO)
	ch = logging.StreamHandler()
	ch.setLevel(logging.INFO)
	formatter = logging.Formatter('%(asctime)s - %(name)s - %(funcName)s - \
	%(levelname)s - %(message)s')
	ch.setFormatter(formatter)
	LOG.addHandler(ch)


def default_distance(options):
	if os.path.splitext(options.partner)[-1] == '.mol2':
		options.distance = 3.5
	elif os.path.splitext(options.partner)[-1] == '.pdb':
		options.distance = 5.0
	return options.distance

# ==============================================================================

if __name__ == "__main__":
	parser = argparse.ArgumentParser(description = 'Identify interface/pocket residues')
	parser.add_argument('target', \
	help = 'Input [.mol2 or .pdb file]: protein target')
	parser.add_argument('partner', \
	help = 'Input [.pdb file]: protein partner or ligand; OR [.mol2 file]: cavity')
	parser.add_argument('-d', dest = 'distance', type = float, \
	help = 'Input [float or integer]: distance threshold \
	{by default, target-protein/ligand = 5A; target-cavity = 3.5A}')
	options = parser.parse_args()
	
	if options.distance is None:
		default_distance(options)
	
	setlogger()
	main(options.target, options.partner, options.distance)
