# -*- coding: utf-8 -*-

"""
Author: Alexandra Moine-Franel
Date: March 2018
Version: 3



IDENTIFY PROTEIN STRUCTURE WITH COMPOUNDS (with or without including ions)
INVOLVED IN PROTEIN-PROTEIN INTERACTION FROM PDBe


Usage: [Script.py] [PDBe] [IONS]
------------------------------------------------------------------
Arguments:
	[PDBe]:
	Input (json format)
	PDBe (with [assembly_composition], [compound_id] and [uniprot_id] included)

	[IONS]:
	Input (txt format)
	Compounds considered as ions 

Return:
	[list(protein)]:
	Output (txt format)
	Proteins with compounds associated with a protein/protein complex PDB IDs

	[json(protein)]:
	Output (json format)
	Proteins with compounds associated with a protein/protein complex dataset

	Compounds:
	---------
	** EXAMPLE: [u'FHB : 3-FLUORO-4-HYDROXYBENZOIC ACID']
			or  [u'FHB : 3-FLUORO-4-HYDROXYBENZOIC ACID', u'FE : FE (III) ION']


    [list(protein)]:
    Output (txt format)
    Proteins with compounds associated with a protein/protein complex PDB IDs
    IONS ONLY EXCLUDED

    [json(protein)]:
    Output (json format)        
    Proteins with compounds associated with a protein/protein complex dataset
    IONS ONLY EXCLUDED

	Compounds:
	---------
	** EXAMPLE: [u'FE : FE (III) ION']
	


!Warning!:
------- 
PDBs with unique and repeated assembly composition annotation are both included!
[unique annotation: 'protein/protein complex']
[repeated annotation: 'protein/protein complex, protein/protein complex']
"""

# =============================================================================

import os
import sys
import argparse
import logging
import json

# =============================================================================

LOG = logging.getLogger('Get PPC PDB IDs with ligand')
FILENAME_with_ions = 'PPC-PROT-COMPOUNDS_3-26-18PDBe'
FILENAME_without_ions = 'PPC-PROT-COMPOUNDS_noIONS_3-26-18PDBe'

XRAY = ['X-ray diffraction', 'X-ray powder diffraction']

RESOLUTION = 3.5
DIFFR = 0.05

# =============================================================================

def main(pdbe, ion):
	with open(pdbe) as infile:
		structure = json.load(infile)

		# Get protein/protein complex (PPC)
		ppc = filter_rfactor(filter_resolution(filter_annotation(
		get_ppc(structure))))
		to_txt(get_id(ppc), 'PPC_3-26-18PDBe')

		# Get protein structure with [compounds]
		prot_compound = filter_rfactor(filter_resolution(filter_annotation(
		get_with_compound(get_prot(structure)))))
		
	# Collect PPC UniProt
	ppc_uniprot = get_uniprot(ppc)

	# Get PPC-associated proteins with compounds
	ppc_ligand = get_prot_compound_ppc(prot_compound, ppc_uniprot)
	to_json(ppc_ligand, FILENAME_with_ions)
	to_txt(get_id(ppc_ligand), FILENAME_with_ions)

	# Get PPC-associated proteins with compounds (ions only not included)
	ions = []
	with open(ion, 'r') as ion_file:
		for i in ion_file:
			ions.append(i.strip().upper())

	prot_compound_without_ions = get_without_ions(prot_compound, ions)
	ppc_ligand_without_ions = get_prot_compound_ppc(prot_compound_without_ions, ppc_uniprot)
	to_json(ppc_ligand_without_ions, FILENAME_without_ions)
	to_txt(get_id(ppc_ligand_without_ions), FILENAME_without_ions)


	if os.path.exists('{}.txt'.format(FILENAME_with_ions)) and \
	os.path.exists('{}.txt'.format(FILENAME_without_ions)):
		LOG.info('Finished!')
	else:
		LOG.error('Warning! [Outputs] Not found!')
		sys.exit(1)



def get_ppc(pdbe):
	ppc_unique = []
	ppc_repeated = []
	for i in xrange(len(pdbe)):
		if 'assembly_composition' in pdbe[i].keys():
			assembly_composition = pdbe[i]['assembly_composition']
			if len(assembly_composition) == 1 and \
			assembly_composition == ['protein/protein complex']:
				ppc_unique.append(pdbe[i])
			elif len(assembly_composition) > 1:
				if assembly_composition.count('protein/protein complex') >= 1 and \
				assembly_composition.count(
				assembly_composition[0]) == len(assembly_composition):
					ppc_repeated.append(pdbe[i])
	ppc_all = ppc_unique + ppc_repeated
	return ppc_all


def get_prot(pdbe):
	prot_unique = []
	prot_repeated = []
	for i in xrange(len(pdbe)):
		if 'assembly_composition' in pdbe[i].keys():
			assembly_composition = pdbe[i]['assembly_composition']
			if len(assembly_composition) == 1 and \
			assembly_composition == ['protein structure']:
				prot_unique.append(pdbe[i])
			elif len(assembly_composition) > 1:
				if assembly_composition.count('protein structure') >= 1 and \
				assembly_composition.count(
				assembly_composition[0]) == len(assembly_composition):
					prot_repeated.append(pdbe[i])
	prot_all = prot_unique + prot_repeated
	return prot_all


def filter_annotation(pdbe):
	annotation = []
	for i in xrange(len(pdbe)):
		if len(pdbe[i]['experimental_method']) == 1:
			annotation.append(pdbe[i])
	return annotation

def filter_resolution(pdbe):
	quality = []
	for i in xrange(len(pdbe)):
		if len([e for e in pdbe[i]['experimental_method'] if e in XRAY]) == 1:
			if 'resolution' in pdbe[i].keys() and \
			float(pdbe[i]['resolution']) <= RESOLUTION:
				quality.append(pdbe[i])
	return quality

def filter_rfactor(pdbe):
	quality = []
	for i in xrange(len(pdbe)):
		if len([e for e in pdbe[i]['experimental_method'] if e in XRAY]) == 1:
			if 'r_free' in pdbe[i].keys() and 'r_factor' in pdbe[i].keys():
				if float(pdbe[i]['r_free'] - pdbe[i]['r_factor']) <= abs(DIFFR):
					quality.append(pdbe[i])
	return quality


def get_with_compound(pdbe):
	compound = []
	for i in xrange(len(pdbe)):
		if 'compound_id' in pdbe[i].keys():
			compound.append(pdbe[i])
	return compound


def get_uniprot(pdbe):
	uniprot = []
	for i in xrange(len(pdbe)):
		if 'uniprot_id' in pdbe[i].keys():
			uniprot.extend(pdbe[i]['uniprot_id'])
	return uniprot


def get_prot_compound_ppc(protein, uniprot):
	prot_compound_ppc = []
	for i in xrange(len(protein)):
		if 'uniprot_id' in protein[i].keys():
			if len([e for e in protein[i]['uniprot_id'] if e in uniprot]) >= 1:
				prot_compound_ppc.append(protein[i])
	return prot_compound_ppc


def get_without_ions(protein, ions):
	without_ions = []
	with_ions_not_only = []
	for i in xrange(len(protein)):
		if 'compound_id' in protein[i].keys():
			if len([e for e in protein[i]['compound_id'] if e in ions]) == 0:
				without_ions.append(protein[i])
			elif len([e for e in protein[i]['compound_id'] if e in ions]) >= 1:
				if len([e for e in protein[i]['compound_id'] if e in ions]) \
				!= len(protein[i]['compound_id']):
					with_ions_not_only.append(protein[i])
	print len(without_ions), len(with_ions_not_only)
	compound_ions = without_ions + with_ions_not_only
	return compound_ions


def get_id(pdbe):
	pdb_id = []
	for i in xrange(len(pdbe)):
		pdb_id.append(pdbe[i]['pdb_id'])
	return pdb_id


def to_json(data, outfile):
	with open('{}.json'.format(outfile), 'wb') as jsonfile:
		json.dump(data, jsonfile)


def to_txt(data, outfile):
	with open('{}.txt'.format(outfile), 'wb') as txtfile:
		for pdb_id in data:
			txtfile.write(pdb_id + '\n')



def setlogger():
	LOG.setLevel(logging.INFO)
	ch = logging.StreamHandler()
	ch.setLevel(logging.INFO)
	formatter = logging.Formatter('%(asctime)s - %(name)s - %(funcName)s - \
	%(levelname)s - %(message)s')
	ch.setFormatter(formatter)
	LOG.addHandler(ch)

# =============================================================================

if __name__ == "__main__":
	parser = argparse.ArgumentParser(description = 'Get protein/protein \
	complex PDB IDs from PDBe')
	parser.add_argument('pdbe', help = 'Input [.json file]: \
	PBDe (with assembly and structure quality information included')
	parser.add_argument('ion', help = 'Input [.txt file]: Compounds considered \
	as ions')
	options = parser.parse_args()
	setlogger()
	main(options.pdbe, options.ion)
