# -*- coding: utf-8 -*-

"""
Author: Alexandra Moine-Franel
Date: February 2018
Version: 2



IDENTIFY INTERACTING LIGANDS AND COMPOUNDS OF PROTEIN-PROTEIN COMPLEXES FROM PDBe


Usage: [Script.py] [PDBe]
------------------------------------------------------------------
Argument:
	[PDBe]:
    Input (json format)
    PDBe (with assembly information included)

Return:
	[list(ligands)]:
    Output (txt format)
    Interacting ligands of protein/protein complexes

	[list(compounds)]:
    Output (txt format)	
    Compounds of protein/protein complexes

	
Warning:
------- 
PDBs with unique and repeated assembly composition annotation are both included!
[unique annotation: 'protein/protein complex']
[repeated annotation: 'protein/protein complex, protein/protein complex']
"""

# =============================================================================

import argparse
import logging
import json
import pandas as pd

# =============================================================================

LOG = logging.getLogger('Get ligands and compounds of protein/protein complexes from PDBe')

# =============================================================================

def setlogger():
    LOG.setLevel(logging.INFO)
    ch = logging.StreamHandler()
    ch.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(funcName)s - %(levelname)s - %(message)s')
    ch.setFormatter(formatter)
    LOG.addHandler(ch)



def main(pdbe):
	structure = pd.read_json(pdbe, orient = 'columns')
	ligand_to_txt(get_ligand(get_ppc_data(structure)))
	compound_to_txt(get_compound(get_ppc_data(structure)))

def get_ppc_data(pdbe):
	ppc_unique_data = []
	ppc_repeated_data = []
	for i in xrange(len(pdbe['grouped']['pdb_id']['groups'])):
		assembly_composition = pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0]['assembly_composition']
		if len(assembly_composition) == 1 and assembly_composition == ['protein/protein complex']:
			ppc_unique_data.append(pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0])
		elif len(assembly_composition) > 1:
			if assembly_composition.count('protein/protein complex') >= 1 and \
			assembly_composition.count(assembly_composition[0]) == len(assembly_composition):
				ppc_repeated_data.append(pdbe['grouped']['pdb_id']['groups'][i]['doclist']['docs'][0])
	ppc_all_data = ppc_unique_data + ppc_repeated_data
	return ppc_all_data



def get_ligand(ppc):
	ppc_ligand = []
	unique_ligand = []
	for i in xrange(len(ppc)):
		if 'interacting_ligands' in ppc[i].keys():
			ppc_ligand.append(ppc[i]['interacting_ligands'][0].split(' :')[0])

	for j in ppc_ligand:
		if j not in unique_ligand:
			unique_ligand.append(j)
			
	#print len(unique_ligand)
	return unique_ligand



def get_compound(ppc):
        ppc_compound = []
        unique_compound = []
	for i in xrange(len(ppc)):
                if 'compound_id' in ppc[i].keys():
			ppc_compound.extend(ppc[i]['compound_id'])

	for j in ppc_compound:
		if j not in unique_compound:
			unique_compound.append(j)

	#print len(unique_compound)
	return unique_compound


def ligand_to_txt(ligand):
	with open('PPC_LIGANDS_2-20-18PDBe.txt', 'wb') as txtfile:
		for lig in ligand:
			txtfile.write(lig + '\n')

def compound_to_txt(compound):
        with open('PPC_COMPOUNDS_2-20-18PDBe.txt', 'wb') as txtfile:
                for c in compound:
                        txtfile.write(c + '\n')

# ==============================================================================

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description = 
					'Get ligands and compounds of protein/protein complexes from PDBe')
    parser.add_argument('pdbe', help = 'Input [.json file]: \
					PBDe (with interacting ligand and compound information included')
    options = parser.parse_args()
    setlogger()
    main(options.pdbe)
